unit
  UColeco;

interface

uses
  SysUtils, Dialogs, Classes, Windows, Forms, UDIBSurf, UZ80Var, UZ80, UGameScr, USprite, UVDP;

var
  oVDP: TVDP;

  function  ColecoInByte(nPort: Word): Byte; pascal;
  procedure ColecoOutByte(nEnder, nValue: Word); pascal;
  function  ColecoGetByte(nEnder: Word): Byte; pascal;
  procedure ColecoPutByte(nEnder, nValue: Word); pascal;
  procedure ColecoInterrupt; pascal;
  procedure ColecoInitDriver;
  procedure ColecoDestroyDriver;
  procedure ColecoLoadCart;

implementation

var
  aSpecKey: array[0..7] of Byte;

procedure ColecoPutByte(nEnder, nValue: Word);
begin
  if (nEnder>$5FFFF) and (nEnder<$8000) then 
    aZ80Mem[$6000+(nEnder and $03FFF)] := nValue
  else
    aZ80Mem[nEnder] := nValue;
end;


function ColecoGetByte(nEnder: Word): Byte;
begin
  if (nEnder>$5FFFF) and (nEnder<$8000) then 
    ColecoGetByte := aZ80Mem[$6000+(nEnder and $03FFF)]
  else
    ColecoGetByte := aZ80Mem[nEnder]
end;

function ColecoInByte(nPort: Word): Byte;
var
  nResult: Byte;
begin
  nResult := $FF;
  case (nPort and $E0) of
    $A0: begin
      if (nPort and $01)<>0 then begin
        nResult := oVDP.nVDPStatus;
        oVDP.nVDPStatus := oVDP.nVDPStatus and $5F;
        oVDP.nVKey := 1;
      end
      else begin
        nResult := oVDP.ReadVRAM;
      end;
    end
  end;
  ColecoInByte := nResult;
end;


procedure ColecoOutByte(nEnder, nValue: Word);
begin
  case (nEnder and $E0) of
    $A0: begin
      if (nEnder and $01)<>0 then begin
        if oVDP.nVKey<>0 then begin
          oVDP.nVR := nValue;
          Dec(oVDP.nVKey);
        end
        else begin
          Inc(oVDP.nVKey);
          case nValue and $C0 of
             $80: oVDP.OutByte(nValue and $07, oVDP.nVR);
             $40: oVDP.nWAddr := oVDP.nVR or ((nValue and $3F) shl 8);
             $00: oVDP.nRAddr := oVDP.nVR or (nValue shl 8);
          end;
        end
      end
      else begin
        if (oVDP.nVKey<>0) then begin
          oVDP.WriteVRAM(nValue)
        end;
      end;
    end;
  end;
end;


procedure VerKey(lKey: Boolean; var I: Byte; J: Byte);
begin
  if lKey then
    I := I and J
  else
    I := I or not J;
end;

procedure ColecoInterrupt;
var
  I, J: Integer;
begin
  frmGameScreen.Caption := IntToStr(oVDP.nScrMode);

  I := oVDP.Interrupt;

  aProcs[1].nPendingIRQ := $00000000;
  if I=$66 then begin
    aProcs[1].nPendingNMI := 1;
  end;


  lDraw := true;
  if lDraw then begin
    lDraw := false;
    if lDoubleScreen then
      CopySurfaceToScreen2x(frmGameScreen.Canvas.Handle)
    else
      CopySurfaceToScreen(frmGameScreen.Canvas.Handle);
  end;
end;

procedure ColecoInitDriver;
var
  I: Integer;
begin
  oVDP.Init(@aVDPInitColeco, 16);
end;


procedure ColecoDestroyDriver;
begin
  oVDP.Done;
end;


function GetByte(var F: File): Byte;
var
  I: Byte;
begin
  BlockRead(F, I, 1);
  GetByte := I;
end;

function Get2(var F: File): Word;
var
  l, h: Word;
begin
  l := GetByte(F);
  h := GetByte(F);
  Get2 := (h shl 8) or l;
end;



procedure ColecoLoadCart;
var
  oOpenDialog: TOpenDialog;
  oSnap: File;
  I: Integer;
  nByte: Byte;
begin
  oOpenDialog := TOpenDialog.Create(nil);
  oOpenDialog.DefaultExt := 'COL';
  oOpenDialog.Filter := 'Coleco SnapShots|*.col';
  oOpenDialog.FileName := oROMSInfo.cCart+'*.col';
  oOpenDialog.Title := 'Coleco ROMs loader';

  if oOpenDialog.Execute then begin
    Assign(oSnap, oOpenDialog.FileName);
    {$I-}
    Reset(oSnap, 1);
    {$I+}
    if IOResult<>0 then Exit;
    BlockRead(oSnap, aZ80Mem[$8000], FileSize(oSnap));
    Close(oSnap);
  end;
  oOpenDialog.Free;
  {$I-}
  ChDir(ExtractFilePath(Application.ExeName));
  {$I+}
  if IOResult<>0 then ShowMessage('Failed changing directory...');
end;



end.