unit ULoadROM;

interface

uses
  UStrings;

type
  TRomInfo = record
    cRomName: ShortString;
    nLocation: LongInt;
    nType: Integer;
  end;
  ARomInfo = array[1..100] of TRomInfo;
  PRomInfo = ^ARomInfo;


  PRomsInfo = ^TRomsInfo;
  TRomsInfo = object
    cColorPROM,
    cTitle,
    cDriver: String;
    nMemGr: LongInt;
    nStarsOn,
    nCharSet,
    nSpriteRAM,
    nSprites,
    nDrvTag,
    nCountRom: Integer;
    oRomInfo: PRomInfo;
    nMaxDirectRead,
    nMaxRom,
    nBulletRAM,
    nAttrRAM, nVRAM,
    nNMIAddr,
    nNMIEnable,
    nChars,
    nCharOffset,
    nDeltaPlane: Integer;
    nW, nH: Integer;
    cCart: String;
    lCart: Boolean;
    constructor LoadInfo(cFileName: String);
    function LoadRoms(cDir: String; P1, P2: Pointer; var cError: String): Boolean;
    destructor Free;
  end;

  function HexToInt(S: String): LongInt;

implementation


 function HexToInt(S: String): LongInt;
 const
   cHexDig: String[16] = '0123456789ABCDEF';
 var
   nMult,
   nAux: LongInt;
   I, nPos: Integer;
 begin
   S := AllTrim(S);
   nAux := 0;
   nMult := 1;
   for i := Length(S) downto 1 do begin
     nPos := Pos(S[i], cHexDig);
     if nPos<>0 then
       nAux := nAux+(nPos-1)*nMult;
     nMult := nMult*16;
   end;
   HexToInt := nAux;
 end;


constructor TROMsInfo.LoadInfo(cFileName: String);
var
  F: System.Text;
  cAux,
  cLine: String;
  nType, nPos: Integer;
begin
  cDriver := '';
  cTitle := '';
  nStarsOn := 0;
  nCharSet := 0;
  nMaxRom := 0;
  nMaxDirectRead := 0;
  nSprites := 0;
  nCountRom := 0;
  nSpriteRAM := 0;
  nBulletRAM := 0;
  nAttrRAM := 0;
  nVRAM := 0;
  nDrvTag := 0;
  cCart := '';
  lCart := false;
  cColorPROM := '';
  Assign(F, cFileName);
  Reset(F);
  while not Eof(F) do begin
    Readln(F, cLine);
    if cLine<>'' then begin
      if cLine[1] in ['0'..'9', 'A'..'F'] then
        Inc(nCountRom);
      if Upper(Copy(cLine, 1, 5))=Upper('#grf=') then Inc(nCountRom);
      if Upper(Copy(cLine, 1, 6))=Upper('#vram=') then begin
        nMemGr := HexToInt(Copy(cLine, 7, 255));
      end;
      if Upper(Copy(cLine, 1, 9))=Upper('#starson=') then begin
        nStarsOn := HexToInt(Copy(cLine, 10, 255));
      end;
      if Upper(Copy(cLine, 1, 9))=Upper('#charset=') then begin
        nCharSet := HexToInt(Copy(cLine, 10, 255));
      end;
      if Upper(Copy(cLine, 1, 9))=Upper('#sprites=') then begin
        nSprites := HexToInt(Copy(cLine, 10, 255));
      end;
      if Upper(Copy(cLine, 1, 8))=Upper('#driver=') then begin
        cDriver := Copy(cLine, 9, 255);
      end;
      if Upper(Copy(cLine, 1, 7))=Upper('#title=') then begin
        cTitle := Copy(cLine, 8, 255);
      end;
      if Upper(Copy(cLine, 1, 7))=Upper('#chars=') then begin
        nChars := HexToInt(Copy(cLine, 8, 255));
      end;
      if Upper(Copy(cLine, 1, 12))=Upper('#charoffset=') then begin
        nCharOffset := HexToInt(Copy(cLine, 13, 255));
      end;
      if Upper(Copy(cLine, 1, 11))=Upper('#colorprom=') then begin
        cColorPROM := Copy(cLine, 12, 255);
      end;
      if Upper(Copy(cLine, 1, 11))=Upper('#cartridge=') then begin
        lCart := true;
        cCart := Copy(cLine, 12, 255);
        if cCart<>'' then
          if cCart[Length(cCart)]<>'\' then cCart := cCart+'\';
      end;
      if Upper(Copy(cLine, 1, 12))=Upper('#deltaplane=') then begin
        nDeltaPlane := HexToInt(Copy(cLine, 13, 255));
      end;
      if Upper(Copy(cLine, 1, 9))=Upper('#screenw=') then begin
        nW := HexToInt(Copy(cLine, 10, 255));
      end;
      if Upper(Copy(cLine, 1, 9))=Upper('#screenh=') then begin
        nH := HexToInt(Copy(cLine, 10, 255));
      end;
      if Upper(Copy(cLine, 1, 8))=Upper('#maxrom=') then begin
        nMaxRom := HexToInt(Copy(cLine, 9, 255));
      end;
      if Upper(Copy(cLine, 1, 15))=Upper('#maxdirectread=') then begin
        nMaxDirectRead := HexToInt(Copy(cLine, 16, 255));
      end;
      if Upper(Copy(cLine, 1, 11))=Upper('#nmienable=') then begin
        nNMIEnable := HexToInt(Copy(cLine, 12, 255));
      end;
      if Upper(Copy(cLine, 1, 8))=Upper('#nmiint=') then begin
        nNMIAddr := HexToInt(Copy(cLine, 9, 255));
      end;
      if Upper(Copy(cLine, 1, 9))=Upper('#attrram=') then begin
        nAttrRAM := HexToInt(Copy(cLine, 10, 255));
      end;
      if Upper(Copy(cLine, 1, 11))=Upper('#bulletram=') then begin
        nBulletRAM := HexToInt(Copy(cLine, 12, 255));
      end;
      if Upper(Copy(cLine, 1, 11))=Upper('#spriteram=') then begin
        nSpriteRAM := HexToInt(Copy(cLine, 12, 255));
      end;
      if Upper(Copy(cLine, 1, 6))=Upper('#vram=') then begin
        nVRAM := HexToInt(Copy(cLine, 7, 255));
      end;
      if Upper(Copy(cLine, 1, 8))=Upper('#drvtag=') then begin
        nDrvTag := HexToInt(Copy(cLine, 8, 255));
      end;
    end;

  end;
  Close(F);

  GetMem(oRomInfo, nCountRom*SizeOf(TRomInfo));

  nCountRom := 0;
  Assign(F, cFileName);
  Reset(F);
  while not Eof(F) do begin
    Readln(F, cLine);
    if cLine<>'' then begin
      nType := 0;

      if Upper(Copy(cLine, 1, 5))=Upper('#grf=') then begin
        nType := 1;
        Delete(cLine, 1, 5);
      end;

      if cLine[1] in ['0'..'9', 'A'..'F'] then begin
        Inc(nCountRom);
        nPos := Pos('~', cLine);
        if nPos<>0 then begin
          oRomInfo^[nCountRom].nLocation := HexToInt(Copy(cLine, 1, nPos-1));
          oRomInfo^[nCountRom].cRomName := Copy(cLine, nPos+1, 255);
          oRomInfo^[nCountRom].nType := nType;
        end;
      end;
    end;
  end;
  Close(F);
end;

function TRomsInfo.LoadRoms(cDir: String; P1, P2: Pointer; var cError: String): Boolean;
type
  TByteArray = array[0..1024] of byte;
  PByteArray = ^TByteArray;
var
  cFileName: String;
  F: File;
  PAux: PChar;
  I: Integer;
begin
  cError := '';

  for i := 1 to nCountRom do begin
    if oRomInfo^[i].nType=0 then
      PAux := P1
    else
      PAux := P2;
    cFileName := cDir+oRomInfo^[i].cRomName;
    Assign(F, cFileName);
    {$I-}
    Reset(F, 1);
    {$I+}
    if IOResult<>0 then begin
      cError := cError+cFileName;
      LoadRoms := false;
      Exit;
    end;
    BlockRead(F, PAux[oRomInfo^[i].nLocation], FileSize(F));
    Close(F);
  end;
  LoadRoms := true;
end;

destructor TRomsInfo.Free;
begin
  FreeMem(oRomInfo, nCountRom*SizeOf(TRomInfo));
end;

end.
