unit
  UMAMEChr;

interface

  uses
    USprite;

  const
    ORIENTATION_FLIP_X  = $80;
    ORIENTATION_FLIP_Y  = $40;
    ORIENTATION_SWAP_XY = $20;

  type
    Int = LongInt;

    PMachine = ^TMachine;
    TMachine = record
      orientation: Int;
    end;

    TByte = array[0..65500] of Byte;
    PByte = ^TByte;

    PBitMapData = ^TBitMapData;
    TBitMapData = record
      line: PByte;
    end;

    PGFXLayout = ^TGFXLayout;
    TGFXLayout = record
      width,height: Int;               (* width and height of chars/sprites *)
      total: Int;                      (* total numer of chars/sprites in the rom *)
      planes: Int;                     (* number of bitplanes *)
      planeoffset: array[0..7] of Int; (* start of every bitplane *)
      xoffset: array[0..63] of Int;    (* coordinates of the bit corresponding to the pixel *)
      yoffset: array[0..63] of Int;    (* of the given coordinates *)
      charincrement: Int;              (* distance between two consecutive characters/sprites *)
    end;

   PGfxElement = ^TGfxElement;
   TGfxElement = record
     width,height: Int;
     gfxdata: PSpriteSet;        (* graphic data *)
     total_elements: Int;    (* total number of characters/sprites *)
     color_granularity: Int; (* number of colors for each color code *)
     colortable: Pointer;        (* map color codes to screen pens *)
     total_colors, nStartPalette: Int;
     cName: String[31];
   end;
   TArrayGfxElement = array[0..1023] of PGfxElement;
   PArrayGfxElement = ^TArrayGfxElement;


procedure FreeGFX(var gfx: PGFXElement);
function DecodeGFX(cName: String; nStartPalette: Integer; oMachine: PMachine; src: PByte; gl: PGfxLayout): PGFXElement;


implementation

function readbit(src: PByte; bitnum: Int): Int;
begin
  readbit := (src^[bitnum div 8] shr (7 - (bitnum mod 8))) and 1;
end;


procedure DecodeChar(oMachine: PMachine; gfx: PGfxElement; num: Int;
                     src: PByte; gl: PGfxLayout);
var
  temp,
  xoffs,
  yoffs,
  offs,
  x,
  y,
  nPlane: Int;
  dp: PByte;
begin
  for nPlane := 0 to gl^.planes-1 do begin
    offs := num * gl^.charincrement + gl^.planeoffset[nPlane];
    for y := 0 to gfx^.height-1 do begin
      dp := @gfx^.gfxdata^[num]^.aData[y*gfx^.width];
      for x := 0 to gfx^.width-1 do begin
        if (nPlane=0) then
          dp^[x] := 0
	else
          dp^[x] := dp^[x] shl 1;

        xoffs := x;
        yoffs := y;

        if (oMachine^.orientation and ORIENTATION_FLIP_X)<>0 then
          xoffs := gfx^.width-1 - xoffs;

        if (oMachine^.orientation and ORIENTATION_FLIP_Y)<>0 then
          yoffs := gfx^.height-1 - yoffs;

        if (oMachine^.orientation and ORIENTATION_SWAP_XY)<>0 then begin
          temp := xoffs;
          xoffs := yoffs;
          yoffs := temp;
        end;
        dp^[x] := dp^[x]+readbit(src,offs + gl^.yoffset[yoffs] + gl^.xoffset[xoffs]);
      end;
    end;
  end;
end;


procedure FreeGFX(var gfx: PGFXElement);
var
  I: Int;
begin
  for i := 0 to gfx^.total_elements-1 do
    FreeMem(gfx^.gfxdata^[i], 2*SizeOf(Int)+gfx^.height*gfx^.width);
  FreeMem(gfx^.gfxdata, gfx^.total_elements*SizeOf(PSprite));
  FreeMem(gfx, SizeOf(TGFXElement));
end;


function DecodeGFX(cName: String; nStartPalette: Integer; oMachine: PMachine; src: PByte; gl: PGfxLayout): PGFXElement;
var
  C: Int;
  gfx: PGFXElement;
  oSpriteSet: PSpriteSet;
begin
  GetMem(gfx, SizeOf(TGFXElement));
  if (oMachine^.orientation and ORIENTATION_SWAP_XY)<>0 then begin
    gfx^.width := gl^.height;
    gfx^.height := gl^.width;
  end
  else begin
    gfx^.width := gl^.width;
    gfx^.height := gl^.height;
  end;
  GetMem(oSpriteSet, gl^.total*SizeOf(PSprite));

  gfx^.total_elements := gl^.total;
  gfx^.color_granularity := 1 shl gl^.planes;
  gfx^.gfxdata := oSpriteSet;
  for c := 0 to gl^.total-1 do begin
    GetMem(oSpriteSet^[c], 2*SizeOf(Int)+gfx^.height*gfx^.width);
    oSpriteSet^[c]^.W := gfx^.width;
    oSpriteSet^[c]^.H := gfx^.height;
    decodechar(oMachine, gfx, c, src, gl);
  end;
  gfx^.cName := cName;
  gfx^.nStartPalette := nStartPalette;
  DecodeGFX := gfx;
end;


end.