unit
  UPengo;

interface

uses
  Classes, Windows, Forms, UDIBSurf, UZ80Var, UZ80, UGameScr, USprite, SysUtils;

  procedure PengoCreateScreen;
  procedure PengoCharSet(aGraph: PMemory);
  function PengoGetByte(nEnder: Word): Byte; pascal;
  procedure PengoPutByte(nEnder, nValue: Word); pascal;
  procedure PengoInterrupt; pascal;
  procedure PengoDestroyDriver;
  procedure PengoConvertColor(aPROM: PByteArray);
  procedure PengoInitDriver;


implementation

const
  aPengoPROM: array[0..31+4] of Byte =
    ($00,$00,$00,$F6,$00,$16,$C0,$3F,$00,$D8,$07,$3F,$00,$C0,$C4,$07,
     $00,$C0,$A0,$07,$00,$00,$00,$07,$00,$F6,$07,$F0,$00,$76,$07,$C6,
     $00,$88,$cc,$ff);
  aUniwarsPROM: array[0..31+4] of Byte = (
   	$00,$e8,$17,$3f,$00,$2f,$87,$20,$00,$ff,$3f,$38,$00,$83,$3f,$06,
	$00,$dc,$1f,$d0,$00,$ef,$20,$96,$00,$3f,$17,$f0,$00,$3f,$17,$14,
        $00,$88,$cc,$ff);
  aWarOfBugsPROM: array[0..31+4] of Byte = (
        $00,$FF,$07,$C0,$00,$07,$C0,$3F,$00,$38,$07,$C0,$00,$07,$C0,$38,
	$00,$3F,$38,$07,$00,$C0,$3F,$07,$00,$F8,$07,$38,$00,$C0,$38,$C7,
        $00,$88,$cc,$ff);

var
  aStars: array[0..MAX_STARS-1] of record
    x,y,code,col: Integer;
  end;


procedure PengoCharSet(aGraph: PMemory);
var
  nColor: Integer;
  I, nCharSet, nSprite, nPixColor, nEnder, nLinSpr, nColSpr: Integer;
  nDeltaCharSet,
  nDeltaPlane: Integer;
  lInvertX, lInvertY: boolean;

  function XY(nLinSpr, nColSpr, nDY, nDX: Integer): Integer;
  begin
    if lInvertX then nColSpr := 7-nColSpr;
    if lInvertY then nLinSpr := 7-nLinSpr;
    XY := (nLinSpr+nDY)*16+nColSpr+nDX;
  end;

begin
  nDeltaCharSet := $1000;
  nDeltaPlane := $800;
  lInvertY := true;
  lInvertX := true;

  GetMem(aCharSet, (256*oROMSInfo.nCharSet)*SizeOf(aCharSet^[0]));

    for I := 0 to (256*oROMSInfo.nCharSet)-1 do begin
      GetMem(aCharSet^[I], 2*SizeOf(Integer)+8*8);
      aCharSet^[I]^.W := 8;
      aCharSet^[I]^.H := 8;
      nCharSet := I div 256;
      nSprite := I mod 256;
      nColor := nSprite shr 5;
      for nLinSpr := 0 to 7 do begin
        for nColSpr := 0 to 7 do begin
          nEnder := (nSprite*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aGraph[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aGraph[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;

          if lInvertX then
            if lInvertY then
              aCharSet^[I]^.aData[7-nLinSpr+8*(7-nColSpr)] := nColor
            else
              aCharSet^[I]^.aData[7-nLinSpr+8*nColSpr] := nColor
          else
            if lInvertY then
              aCharSet^[I]^.aData[nLinSpr+8*(7-nColSpr)] := nColor
            else
              aCharSet^[I]^.aData[nLinSpr+8*nColSpr] := nColor
        end;
      end;
    end;


  GetMem(aSprites, oROMSInfo.nSprites*SizeOf(aSprites^[0]));
  if oROMSInfo.nSprites<>0 then begin
    for i := 0 to oROMSInfo.nSprites-1 do begin
      nSprite := 4*i;
      nCharSet := nSprite div 256;
      nSprite := nSprite mod 256;

      GetMem(aSprites^[i], 2*SizeOf(Integer)+16*16);
      aSprites^[i].W := 16;
      aSprites^[i].H := 16;

      for nLinSpr := 0 to 7 do
        for nColSpr := 0 to 7 do begin
          nEnder := ((nSprite+0)*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aGraph[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aGraph[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          aSprites^[i]^.aData[XY(nColSpr, nLinSpr, 0, 8)] := nColor;
        end;

      for nLinSpr := 0 to 7 do
        for nColSpr := 0 to 7 do begin
          nEnder := ((nSprite+1)*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aGraph[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aGraph[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          aSprites^[i]^.aData[XY(nColSpr, nLinSpr, 8, 8)] := nColor;
        end;

      for nLinSpr := 0 to 7 do
        for nColSpr := 0 to 7 do begin
          nEnder := ((nSprite+2)*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aGraph[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aGraph[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          aSprites^[i]^.aData[XY(nColSpr, nLinSpr, 0, 0)] := nColor;
        end;

      for nLinSpr := 0 to 7 do
        for nColSpr := 0 to 7 do begin
          nEnder := ((nSprite+3)*8+nLinSpr)+nCharSet*nDeltaCharSet;
          nPixColor := Bit(aGraph[nEnder+nDeltaPlane], nColSpr);
          nPixColor := (nPixColor shl 0) or
                       (Bit(aGraph[nEnder+$0000], nColSpr) shl 1);
          nColor := nPixColor;
          aSprites^[i]^.aData[XY(nColSpr, nLinSpr, 8, 0)] := nColor;
        end;
    end;
  end;

end;


function PengoGetByte(nEnder: Word): Byte;
var
  RV: Byte;
begin

  if nEnder=Pengo_IN0_PORT then begin
    rv := 0;
    if (GetAsyncKeyState(VK_LEFT) and $8000)<>0  then rv  := rv or Pengo_IN0_P1_LEFT;
    if (GetAsyncKeyState(VK_RIGHT) and $8000)<>0 then rv  := rv or Pengo_IN0_P1_RIGHT;
    if (GetAsyncKeyState(VK_3) and $8000)<>0     then rv  := rv or Pengo_IN0_COIN1;
    if (GetAsyncKeyState(VK_4) and $8000)<>0     then rv  := rv or Pengo_IN0_COIN2;
    if (GetAsyncKeyState(VK_CONTROL) and $8000)<>0 then rv  := rv or Pengo_IN0_P1_SHOOT;

    if oRomsInfo.nDrvTag=2 then begin
      if (GetAsyncKeyState(VK_UP) and $8000)<>0     then rv  := rv or Pengo_IN0_TEST;
      if (GetAsyncKeyState(VK_DOWN) and $8000)<>0   then rv  := rv or Pengo_IN0_SERVICE;
    end
    else begin
      if (GetAsyncKeyState(VK_F2) and $8000)<>0     then rv  := rv or Pengo_IN0_TEST;
      if (GetAsyncKeyState(VK_F3) and $8000)<>0     then rv  := rv or Pengo_IN0_SERVICE;
    end;
    PengoGetByte := rv;
    exit
  end;

  if nEnder=Pengo_IN1_PORT then begin
    rv := $10;
    if (GetAsyncKeyState(VK_2) and $8000)<>0     then rv  := rv or Pengo_IN1_2P_START;
    if (GetAsyncKeyState(VK_1) and $8000)<>0     then rv  := rv or Pengo_IN1_1P_START;
    PengoGetByte := rv;
    exit
  end;

  if nEnder=Pengo_DSW_PORT then begin
    rv := $00;
    PengoGetByte := $FF;
    exit
  end;



  PengoGetByte := aZ80Mem[nEnder];
end;


procedure PengoPutByte(nEnder, nValue: Word);
begin
  {if (nEnder<Pengo_END_OF_ROM) then exit;}
  if (nEnder=$6000) or (nEnder=$6001) then exit;

  if nEnder>=$4FFF then begin
    if aZ80Mem[nEnder]<>nValue then begin
      lDraw := true;
    end;
  end;

  if nEnder=$6002 then begin
    if oRomsInfo.nDrvTag>=$50 then begin
      nGFXBank := nValue and 1;
    end;
  end;

  aZ80Mem[nEnder] := nValue
end;


procedure PengoCreateScreen;
var
  nChar, nSprite, nEnder, i, j, nty, x, y, c, nDeltaY: LongInt;
  nY, nSX, nSY, nShift, nColor, nTotalX, nOffSetValue, nScanLine, nPos: LongInt;
  nFlip: Byte;
begin
  FillSurface(0);
  if (aZ80Mem[oROMSInfo.nStarsOn] and 1)<>0 then
    for nOffSetValue := 0 to nTotalStars-1 do begin
      x := (aStars[nOffSetValue].x + nStarsScroll div 2) mod 256;
      y := aStars[nOffSetValue].y;
      if ((y and 1) xor ((x shr 4) and 1))<>0 then
        SetPixelSafe(y, x, 10+aStars[nOffSetValue].col)
    end;


  for y := 0 to 31 do begin
    if true then begin
      nY := (31-Y);
    end
    else begin
      nY := Y;
    end;
    nSY := nY*8;
    nShift := aZ80Mem[oROMSInfo.nAttrRAM+2*nY];

    nColor := 10+(aZ80Mem[oROMSInfo.nAttrRAM+2*nY+1] and 7)*4;

    for x := 0 to 31 do begin
      if true then
        nOffSetValue := aZ80Mem[oROMSInfo.nVRAM+(31-x)*32+nY]
      else
        nOffSetValue := aZ80Mem[oROMSInfo.nVRAM+x*32+nY];
      {if nOffSetValue=16 then continue;}
      nSX := (x*8+nShift) mod 256;

      if (nSX>=8) and (nSX<(256-8)) then
        PutTransSprite(nSX, nSY, nColor, aCharSet^[nOffSetValue]);
    end;
  end;

  for i := 0 to 7 do begin
    x := aZ80Mem[oROMSInfo.nBulletRAM+4*i+1];
    if (x<>0) and (x<>255) then begin
      y := 256-aZ80Mem[oROMSInfo.nBulletRAM+4*i+3]-4;
      if oRomsInfo.nDrvTag>=$80 then begin
        for j := 0 to 2 do
          SetPixelSafe(x+j, y, 97+10)
      end
      else begin
        for j := 0 to 2 do
          SetPixelSafe(x, y+j, 97+10)
      end;
    end;
  end;


  for i := 0 to 7 do begin
    x := aZ80Mem[oROMSInfo.nSpriteRAM+4*i];
    if (x>0) and (x<(255-16)) then begin
      nSprite := aZ80Mem[oROMSInfo.nSpriteRAM+4*i+1] and $3F;
      nSprite := nSprite+nGFXBank*64;
      nColor := 10+4*(aZ80Mem[oROMSInfo.nSpriteRAM+4*i+2] and 07);
      if nSprite<>0 then begin
        y := aZ80Mem[oROMSInfo.nSpriteRAM+4*i+3];
        if (y>0) and (y<(255-16)) then begin
          nFlip := 0;
          if (aZ80Mem[oROMSInfo.nSpriteRAM+4*i+1] and $40)<>0 then nFlip := nFlip or $40;
          if (aZ80Mem[oROMSInfo.nSpriteRAM+4*i+1] and $80)<>0 then nFlip := nFlip or $80;
          xPutSprite(X, Y, nColor, nFlip, aSprites^[nSprite]);
        end;
      end;
    end;
  end;
end;


procedure PengoInterrupt;
begin
  if oROMSInfo.nDrvTag<$80 then
    Inc(nStarsScroll);

  PengoCreateScreen;

  if lDoubleScreen then
    CopySurfaceToScreen2x(frmGameScreen.Canvas.Handle)
  else
    CopySurfaceToScreen(frmGameScreen.Canvas.Handle);

  aProcs[1].nPendingIRQ := $01000000;
  if oROMSInfo.nNMIEnable<>0 then
    if aZ80Mem[oROMSInfo.nNMIEnable]<>0 then begin
      aProcs[1].nPendingNMI := 1
    end
  {NullInterrupt;}
end;




procedure PengoConvertColor(aPROM: PByteArray);
var
  nBits: Integer;
  nGenerator: LongInt;
  S, X, Y, I, nBit0, nBit1, nBit2, nVal: Integer;
begin
  for i := 0 to 31 do begin
    nVal := aPROM^[i];
    nBit0 := (nVal shr 0) and $01;
    nBit1 := (nVal shr 1) and $01;
    nBit2 := (nVal shr 2) and $01;
    aPalette[i].R := $21*nBit0+$47*nBit1+$97*nBit2;

    nBit0 := (nVal shr 3) and $01;
    nBit1 := (nVal shr 4) and $01;
    nBit2 := (nVal shr 5) and $01;
    aPalette[i].G := $21*nBit0+$47*nBit1+$97*nBit2;

    nBit0 := 0;
    nBit1 := (nVal shr 6) and $01;
    nBit2 := (nVal shr 7) and $01;
    aPalette[i].B := $21*nBit0+$47*nBit1+$97*nBit2;
  end;

  for i := 0 to 63 do begin
    nBits :=  (i shr 0) and $03;
    aPalette[i+32].R := aPROM^[32+nBits];
    nBits :=  (i shr 2) and $03;
    aPalette[i+32].G := aPROM^[32+nBits];
    nBits :=  (i shr 4) and $03;
    aPalette[i+32].B := aPROM^[32+nBits];
  end;

  aPalette[4].R := 0; aPalette[4].G := 0; aPalette[4].B := $FF;
  aPalette[97].R := $FF; aPalette[97].G := $FF; aPalette[97].B := $FF;

  nTotalStars := 0;
  nGenerator := 0;
  for y := 255 downto 0 do
    for x := 511 downto 0 do begin
      nGenerator := nGenerator shl 1;
      nBit1 := ((not nGenerator) shr 17) and 1;
      nBit2 := (nGenerator shr 5) and 1;
      if (nBit1 xor nBit2)<>0 then begin
        nGenerator := nGenerator or 1;
        if (y>=0) and (y<=255) and ((((not nGenerator) shr 16) and 1)<>0) and ((nGenerator and $FF)=$FF) then begin
          I := not(nGenerator shr 8) and $3F;
          if (I and nTotalStars)<MAX_STARS then begin
            aStars[nTotalStars].x := x;
            aStars[nTotalStars].y := y;
            aStars[nTotalStars].code := i;
            aStars[nTotalStars].col := i+32;
            Inc(nTotalStars);
          end;
        end;
      end;
    end;
end;


procedure PengoDestroyDriver;
var
  I: Integer;
begin
  if oROMSInfo.nCharSet<>0 then begin
    for I := 0 to (256*oROMSInfo.nCharSet)-1 do
      FreeMem(aCharSet^[I], 2*SizeOf(Integer)+8*8);
    FreeMem(aCharSet, (256*oROMSInfo.nCharSet)*SizeOf(aCharSet^[0]));
  end;
  if oROMSInfo.nSprites<>0 then begin
    for i := 0 to oROMSInfo.nSprites-1 do
      FreeMem(aSprites^[i], 2*SizeOf(Integer)+16*16);
    FreeMem(aSprites, oROMSInfo.nSprites*SizeOf(aSprites^[0]));
  end;
end;

procedure PengoInitDriver;
begin
  case oRomsInfo.nDrvTag of
    $00: PengoConvertColor(@aPengoPROM);
    $02: PengoConvertColor(@aWarOfBugsPROM);
    $51, $50: PengoConvertColor(@aUniwarsPROM);
  end;
end;

end.