unit
  USpect;

interface

uses
  SysUtils, Dialogs, Classes, Windows, Forms, UDIBSurf, UZ80Var, UZ80, UGameScr, USprite;

const
  aSpectrumColors: array[0..15] of array[1..3] of Byte =
    ( (* Normal colours *)
     ( $00, $00, $00), ( $00, $00, $cf),
     ( $cf, $00, $00), ( $cf, $00, $cf),
     ( $00, $cf, $00), ( $00, $cf, $cf),
     ( $cf, $cf, $00), ( $cf, $cf, $cf),
     (* Brigth colours *)
     ( $00, $00, $00), ( $00, $00, $ff),
     ( $ff, $00, $00), ( $ff, $00, $ff),
     ( $00, $ff, $00), ( $00, $ff, $ff),
     ( $ff, $ff, $00), ( $ff, $ff, $ff));


  function  SpectrumInByte(nPort: Word): Byte; pascal;
  procedure SpectrumOutByte(nEnder, nValue: Word); pascal;
  function  SpectrumGetByte(nEnder: Word): Byte; pascal;
  procedure SpectrumPutByte(nEnder, nValue: Word); pascal;
  procedure SpectrumInterrupt; pascal;
  procedure SpectrumInitDriver;
  procedure SpectrumDestroyDriver;
  procedure SpectrumLoadCart;

implementation

var
  aSpecKey: array[0..7] of Byte;

procedure SpectrumPutByte(nEnder, nValue: Word);
const
  aBit: array[0..7] of Byte = (128,64,32,16,8,4,2,1);
var
  I, nCol, nLin, nMLin: Integer;
  nAttr, nPaper, nInk: Byte;
begin
  if nEnder<16384 then Exit;

  aZ80Mem[nEnder] := nValue;

  if (nEnder>=$4000) and (nEnder<$5800) then begin
    nLin := ((nEnder shr 8) and $07) or
            ((nEnder shr 2) and $38) or
            ((nEnder shr 5) and $C0);
    nCol := nEnder and $1F;

    nAttr := aZ80Mem[$5800+nCol+(nLin shr 3)*32];
    nPaper := (nAttr shr 3) and $0F;
    nInk := (nAttr and 7) or ((nAttr shr 3) and 8);
    for i := 0 to 7 do
      if (nValue and aBit[i])=0 then
        SetPixelFast(nCol*8+i, nLin, nPaper+10)
      else
        SetPixelFast(nCol*8+i, nLin, nInk+10);
    lDraw := true;
    Exit;
  end;


  if (nEnder>=$5800) and (nEnder<=($5800+24*32-1)) then begin
    nCol := (nEnder-$5800) mod 32;
    nLin := (nEnder-$5800) div 32;
    nCol := ((nLin and $07) shl 5) or
            ((nLin and $18) shl 8) or nCol or $4000;

    for i := 0 to 7 do SpectrumPutByte(nCol+i*256, aZ80Mem[nCol+i*256]);
  end;
end;


function SpectrumGetByte(nEnder: Word): Byte;
begin
  SpectrumGetByte := aZ80Mem[nEnder]
end;

function SpectrumInByte(nPort: Word): Byte;
var
  I, nValue, nTMPValue, TVal, nScanStat: Byte;
begin

  nValue := 0;
  if nPort=$FFFE then begin
    SpectrumInByte := $FF;
    Exit;
  end;

  case (nPort and $FF) of
    $FE: begin
      nValue := $FF;
      for i := 0 to 7 do
        if Bit(nPort shr 8, i)=0 then begin
          nValue := nValue and aSpecKey[i];
        end;
    end;
  end;

  SpectrumInByte := nValue;
end;


procedure SpectrumOutByte(nEnder, nValue: Word);
begin
end;


procedure VerKey(lKey: Boolean; var I: Byte; J: Byte);
begin
  if lKey then
    I := I and J
  else
    I := I or not J;
end;

procedure SpectrumInterrupt;
var
  I, J: Integer;
begin
  VerKey((GetAsyncKeyState(VK_1) and $8000)<>0, aSpecKey[3], $FE);
  VerKey((GetAsyncKeyState(VK_2) and $8000)<>0, aSpecKey[3], $FD);
  VerKey((GetAsyncKeyState(VK_3) and $8000)<>0, aSpecKey[3], $FB);
  VerKey((GetAsyncKeyState(VK_4) and $8000)<>0, aSpecKey[3], $F7);
  VerKey((GetAsyncKeyState(VK_5) and $8000)<>0, aSpecKey[3], $EF);
  VerKey((GetAsyncKeyState(VK_Q) and $8000)<>0, aSpecKey[2], $FE);
  VerKey((GetAsyncKeyState(VK_W) and $8000)<>0, aSpecKey[2], $FD);
  VerKey((GetAsyncKeyState(VK_E) and $8000)<>0, aSpecKey[2], $FB);
  VerKey((GetAsyncKeyState(VK_R) and $8000)<>0, aSpecKey[2], $F7);
  VerKey((GetAsyncKeyState(VK_T) and $8000)<>0, aSpecKey[2], $EF);
  VerKey((GetAsyncKeyState(VK_A) and $8000)<>0, aSpecKey[1], $FE);
  VerKey((GetAsyncKeyState(VK_S) and $8000)<>0, aSpecKey[1], $FD);
  VerKey((GetAsyncKeyState(VK_D) and $8000)<>0, aSpecKey[1], $FB);
  VerKey((GetAsyncKeyState(VK_F) and $8000)<>0, aSpecKey[1], $F7);
  VerKey((GetAsyncKeyState(VK_G) and $8000)<>0, aSpecKey[1], $EF);
  VerKey((GetAsyncKeyState(VK_CONTROL) and $8000)<>0, aSpecKey[0], $FE);
  VerKey((GetAsyncKeyState(VK_Z) and $8000)<>0, aSpecKey[0], $FD);
  VerKey((GetAsyncKeyState(VK_X) and $8000)<>0, aSpecKey[0], $FB);
  VerKey((GetAsyncKeyState(VK_C) and $8000)<>0, aSpecKey[0], $F7);
  VerKey((GetAsyncKeyState(VK_V) and $8000)<>0, aSpecKey[0], $EF);
  VerKey((GetAsyncKeyState(VK_0) and $8000)<>0, aSpecKey[4], $FE);
  VerKey((GetAsyncKeyState(VK_9) and $8000)<>0, aSpecKey[4], $FD);
  VerKey((GetAsyncKeyState(VK_8) and $8000)<>0, aSpecKey[4], $FB);
  VerKey((GetAsyncKeyState(VK_7) and $8000)<>0, aSpecKey[4], $F7);
  VerKey((GetAsyncKeyState(VK_6) and $8000)<>0, aSpecKey[4], $EF);
  VerKey((GetAsyncKeyState(VK_P) and $8000)<>0, aSpecKey[5], $FE);
  VerKey((GetAsyncKeyState(VK_O) and $8000)<>0, aSpecKey[5], $FD);
  VerKey((GetAsyncKeyState(VK_I) and $8000)<>0, aSpecKey[5], $FB);
  VerKey((GetAsyncKeyState(VK_U) and $8000)<>0, aSpecKey[5], $F7);
  VerKey((GetAsyncKeyState(VK_Y) and $8000)<>0, aSpecKey[5], $EF);
  VerKey((GetAsyncKeyState(13) and $8000)<>0, aSpecKey[6], $FE);
  VerKey((GetAsyncKeyState(VK_L) and $8000)<>0, aSpecKey[6], $FD);
  VerKey((GetAsyncKeyState(VK_K) and $8000)<>0, aSpecKey[6], $FB);
  VerKey((GetAsyncKeyState(VK_J) and $8000)<>0, aSpecKey[6], $F7);
  VerKey((GetAsyncKeyState(VK_H) and $8000)<>0, aSpecKey[6], $EF);
  VerKey((GetAsyncKeyState(32) and $8000)<>0, aSpecKey[7], $FE);
  VerKey((GetAsyncKeyState(VK_SHIFT) and $8000)<>0, aSpecKey[7], $FD);
  VerKey((GetAsyncKeyState(VK_M) and $8000)<>0, aSpecKey[7], $FB);
  VerKey((GetAsyncKeyState(VK_N) and $8000)<>0, aSpecKey[7], $F7);
  VerKey((GetAsyncKeyState(VK_B) and $8000)<>0, aSpecKey[7], $EF);

  for i := 0 to 7 do
    for j := 0 to 4 do
      if (aSpecKey[i] and (1 shl j))<>0 then
        SetPixelFast(j*3, 200+2*i, 10+7)
      else
        SetPixelFast(j*3, 200+2*i, 10+1);

  aProcs[1].nPendingIRQ := $00000000;

  lDraw := true;
  if lDraw then begin
    lDraw := false;
    if lDoubleScreen then
      CopySurfaceToScreen2x(frmGameScreen.Canvas.Handle)
    else
      CopySurfaceToScreen(frmGameScreen.Canvas.Handle);
  end;
end;

procedure SpectrumInitDriver;
var
  I: Integer;
begin
  for i := 0 to 15 do begin
     aPalette[i].R := aSpectrumColors[i, 1];
     aPalette[i].G := aSpectrumColors[i, 2];
     aPalette[i].B := aSpectrumColors[i, 3];
  end;
end;


procedure SpectrumDestroyDriver;
begin
end;


function GetByte(var F: File): Byte;
var
  I: Byte;
begin
  BlockRead(F, I, 1);
  GetByte := I;
end;

function Get2(var F: File): Word;
var
  l, h: Word;
begin
  l := GetByte(F);
  h := GetByte(F);
  Get2 := (h shl 8) or l;
end;



procedure SpectrumLoadCart;
var
  oOpenDialog: TOpenDialog;
  oSnap: File;
  I: Integer;
  nByte: Byte;
begin
  oOpenDialog := TOpenDialog.Create(nil);
  oOpenDialog.DefaultExt := 'SNA';
  oOpenDialog.Filter := 'Spectrum SnapShots|*.sna';
  oOpenDialog.FileName := oROMSInfo.cCart+'*.sna';
  oOpenDialog.Title := 'ZXSpectrum Snapshots loader';

  if oOpenDialog.Execute then begin
    Assign(oSnap, oOpenDialog.FileName);
    {$I-}
    Reset(oSnap, 1);
    {$I+}
    if IOResult<>0 then Exit;
    I := getbyte(oSnap);                { 00 }
    aZ80Regs^.HL1.w := get2(oSnap);                 { 01 }
    aZ80Regs^.DE1.w := get2(oSnap);                 { 03 }
    aZ80Regs^.BC1.w := get2(oSnap);                 { 05 }
    aZ80Regs^.AF1.w := get2(oSnap);                 { 07 }
    aZ80Regs^.HL.w := get2(oSnap);                  { 09 }
    aZ80Regs^.DE.w := get2(oSnap);                  { 0B }
    aZ80Regs^.BC.w := get2(oSnap);                  { 0D }
    aZ80Regs^.IY.w := get2(oSnap);                  { 0F }
    aZ80Regs^.IX.w := get2(oSnap);                  { 11 }
    aZ80Regs^.IFF1 := (getbyte(oSnap) shr 2) and 1; { 13 }
    aZ80Regs^.IFF2 := aZ80Regs^.IFF1;
    aZ80Regs^.R := getbyte(oSnap);                  { 14 }
    aZ80Regs^.AF.l := getbyte(oSnap);               { 15 }
    aZ80Regs^.AF.h := getbyte(oSnap);               { 16 }
    aZ80Regs^.SP.w := get2(oSnap);                  { 17 }
    aProcs[1].nIM  := GetByte(oSnap);               { 19 }
    I := GetByte(oSnap);                            { 20 }
    for i := $4000 to $FFFF do begin
      nByte := GetByte(oSnap);
      SpectrumPutByte(i, nByte);
      if (i<=($4000+8192+718)) and ((i mod $20)=0) then begin
        if lDoubleScreen then
          CopySurfaceToScreen2x(frmGameScreen.Canvas.Handle)
        else
          CopySurfaceToScreen(frmGameScreen.Canvas.Handle);
        Application.ProcessMessages;
      end;
    end;
    aZ80Regs^.PC.w := 256*SpectrumGetByte(aZ80Regs^.SP.w+1)+SpectrumGetByte(aZ80Regs^.SP.w);
    Dec(aZ80Regs^.SP.w, 2);
    Close(oSnap);

  end;
  oOpenDialog.Free;
  {$I-}
  ChDir(ExtractFilePath(Application.ExeName));
  {$I+}
  if IOResult<>0 then ShowMessage('Failed changing directory...');
end;


end.