{
 *********************************************************************
 *    Unidade Strings                                                *
 *    ---------------                                                *
 *                                                                   *
 *    Autor : Paulo Sergio Cardoso                                   *
 *    Data  : 03-11-1991                                             *
 *                                                                   *
 *    Contem rotinas gerais de transformacao de numeros reias em     *
 * cadeias string, alem de rotinas gerais de manipulacao de strings. *
 *                                                                   *
 *    Alteracoes                                                     *
 *    ------------------------------------------                     *
 *    Alteracao                                                      *
 *    Autor                                                          *
 *    Data                                                           *
 *    ------------------------------------------                     *
 *                                                                   *
 *********************************************************************



}

Unit
  UStrings;

Interface
{
Uses
  WinDos;
}

Function NumToStr (X:Real;N:Byte):String;
Function Replicate (C:Char;N:Byte):String;
Function Space (N:Byte):String;
Function RealToStr   (X:Real;N:Byte;M:Byte):String;
Function ScientToStr (X:Real;N:Byte):String;
Function sStr   (X:LongInt; N:Byte):String;
Function sStrZero  (X:LongInt; N:Byte):String;
Function AllTrim (S:String):String;
Function Fixa (Frase:String;Qtd:Byte):String;
Function Upper (S:String):String;
Function XorText (Frase:String; XorByte:Byte):String;
Function XorNumeroSerie (Frase:String):String;
function ValNum (cStr:String; var nError: Integer):LongInt;
Function ValReal (cStr:String):Real;
Function TodaysDate:String;
Function TodaysDateTime:String;
Function FormatDate (S:String):String;
Function UnFormatDate (S:String):String;
Function FormatTime (S:String):String;
Procedure CalculaDiaSemana (nDia,nMes,nAno:Integer; Var nDiaS,nTotDias:Integer);
Function CompDateTime (S1,S2:String):Integer;
Function ExtractDate (S:String):String;
Function ExtractTime (S:String):String;
Function ComputeDTime (Date,Time:String):String;
function HexToInt(S: String): LongInt;
function ToHex(nNumber, nSize: LongInt): String;

Implementation

{
 ***************************************************************************
 *          Funcao NumToStr                                                *
 *          ---------------                                                *
 *          Entradas : X - Numero Real                                     *
 *                     N - Tamanho da String de Saida                      *
 *          A funcao devolve uma string contendo o numero X convertido.    *
 *          Se for um numero inteiro, devolve a string sem valores apos    *
 *          a virgula. Se for menor que 1e-5 devolve em notacao cientifica.*
 ***************************************************************************
}
Function NumToStr (X:Real;N:Byte):String;
Var
  S:String;
Begin

If X=0 then Begin
            Str (0:N,S);
            NumToStr := S
            End;

If (Frac(X)<1e-6) and (Int(X)<>0)
   then Begin
        Str (X:N:0,S);
        NumToStr := S;
        Exit;
        End;
If (Abs(X)>1e-3) and (Abs(X)<1e+3)
   then Begin
        Str (X:N:5,S);
        NumToStr := S;
        Exit
        End;

Str (X:N,S);
NumToStr := S
End;


{
 ********************************************************************
 * Funcao Replicate                                                 *
 * ----------------                                                 *
 * Entradas : C - Caractere                                         *
 *            N - Byte                                              *
 * Devolve um string contendo N caracteres, todos eles iguais a C.  *
 ********************************************************************
}
Function Replicate;
Var
  S : String;
  I : Byte;
Begin
S := '';
For I := 1 to N do
    S := S + C;
Replicate := S
End;

Function Space(N:Byte):String;
Begin
Space := Replicate (' ',N);
End;


{
 *********************************************************************
 * Funcao Fixa                                                       *
 * -----------                                                       *
 * Entradas : Frase - String                                         *
 *            Qtd   - Byte                                           *
 * Devolve uma string que contem a Frase truncada em Qtd caracteres. *
 *********************************************************************
}
Function Fixa (Frase:String;Qtd:Byte):String;
Var
  Aux : String;
Begin
Aux := Frase;
While Length(Aux)<Qtd do
      Aux := #32+Aux;
Fixa := Aux
End;


{
 **************************************************************************
 * Funcao RealToStr                                                       *
 * ----------------                                                       *
 * Entradas : X   - Real                                                  *
 *            N,M - Byte                                                  *
 * Devolve uma string contendo a transformacao do numero X em um string.  *
 * A string contera o numero com N caracteres, com M bytes decimais (con- *
 * tando o ponto flutuante).                                              *
 **************************************************************************
}
Function RealToStr (X:Real;N:Byte;M:Byte):String;
Var
  Aux : String;
Begin
Str (X:N:M,Aux);
RealToStr := Aux
End;


{
 **********************************************************************
 *  Funcao ScientToStr                                                *
 *  ------------------                                                *
 *  Entradas : X - Numero Real                                        *
 *             N - Byte                                               *
 *  Devolve uma string de N caracteres contendo o numero X em notacao *
 *  cientifica (9.999E-99).                                           *
 **********************************************************************
}
Function ScientToStr (X:Real;N:Byte):String;
Var
  Aux : String;
Begin
Str (X:N,Aux);
ScientToStr := Aux
End;


{
  *************************************************************
  *  Funcao sStr                                         *
  *  ----------------                                         *
  *  Entradas : X - Inteiro Longo (32 bits)                   *
  *             N - Byte                                      *
  *  Devolve uma string de N bytes contendo o numero X        *
  *************************************************************
}
Function sStr (X:LongInt;N:Byte):String;
Var
  Aux : String;
Begin
Str (X:N,Aux);
sStr := Aux
End;


{
  *************************************************************
  *  Funcao sStrZero                                        *
  *  -----------------                                        *
  *  Entradas : X - Inteiro Longo (32 bits)                   *
  *             N - Byte                                      *
  *  Devolve uma string de N bytes contendo o numero X, colo- *
  *  cando zeros a esquerda se n for maior que o numero de    *
  *  digitos do numero inteiro X                              *
  *************************************************************
}
Function sStrZero (X:LongInt;N:Byte):String;
Var
  Aux : String;
  I   : Byte;
Begin
Str (X:N,Aux);
For i := 1 to Length(Aux) do
    If Aux[i]=' ' then Aux[i] := '0';
sStrZero := Aux
End;

{
 *****************************************************************
 * Funcao AllTrim                                                *
 * --------------                                                *
 * Entradas : S - String                                         *
 * Devolve a String S, retirando todos os espacos a esquerda e a *
 * direita.                                                      *
 *****************************************************************
}
Function AllTrim (S:String):String;
Var
  I : Byte;
Begin
While (Length(S)>0) and (S[1]=#32) do
      Delete (S,1,1);
While (Length(S)>0) and (S[Length(S)]=#32) do
      Delete (S,Length(S),1);
AllTrim := S
End;


Function Upper (S:String):String;
Var
  X : String;
  I : Byte;
Begin
X := S;
For i := 1 to Length(S) do
    X[i] := UpCase(S[i]);
Upper := X
End;


Function XorText (Frase:String; XorByte:Byte):String;
Var
  I : Byte;
Begin
For i := 1 to Length(Frase) do
  if (i mod 2)=0 then
    Frase[i] := Char(Byte(Frase[i]) xor XorByte)
  else
    Frase[i] := Char(Byte(Frase[i]) xor (XorByte+1));
XorText := Frase
End;


Function XorNumeroSerie (Frase:String):String;
Var
  I : Byte;
Begin
For i := 1 to Length(Frase) do
    Frase[i] := Char(Byte(Frase[i]) xor ($70+i));
XorNumeroSerie := Frase
End;

Function ValNum (cStr:String; var nError: Integer):LongInt;
Var
  nAux : LongInt;
Begin
Val (AllTrim(cStr),nAux,nError);
ValNum := nAux
End;

Function ValReal (cStr:String):Real;
Var
  nError : Integer;
  nAux : Real;
Begin
Val (AllTrim(cStr),nAux,nError);
ValReal := nAux
End;



Function FormatDate (S:String):String;
Begin
FormatDate := Copy(S,7,2)+'/'+Copy(S,5,2)+'/'+Copy(S,1,4);
End;

Function TodaysDate:String;
Var
  nDia,nMes,nAno,nDiaS : Word;
Begin
{GetDate (nAno,nMes,nDia,nDiaS);}
TodaysDate := sStrZero(nDia,2)+'/'+sStrZero(nMes,2)+'/'+sStrZero(nAno,4)
End;

Function TodaysDateTime:String;
Var
  nDia,nMes,nAno,nDiaS    : Word;
  nHora,nMin,nSeg,nSeg100 : Word;
Begin
{
GetDate (nAno,nMes,nDia,nDiaS);
GetTime (nHora,nMin,nSeg,nSeg100);
}
TodaysDateTime := sStrZero(nAno,4)+sStrZero(nMes,2)+sStrZero(nDia,2)+
                  sStrZero(nHora,2)+sStrZero(nMin,2)+sStrZero(nSeg,2);
End;

Function CompDateTime (S1,S2:String):Integer;
Begin
If S1>S2 then CompDateTime := 1
         else If S1<S2 then CompDateTime := -1
                       else CompDateTime := 0;
End;


Function UnFormatDate (S:String):String;
Begin
UnFormatDate := Copy(S,7,4)+Copy(S,4,2)+Copy(S,1,2);
End;

Function FormatTime (S:String):String;
Begin
FormatTime := Copy(S,1,2)+'h'+Copy(S,3,2)+'min'
End;


Procedure CalculaDiaSemana (nDia,nMes,nAno:Integer; Var nDiaS,nTotDias:Integer);
Const
   ZELLER_OFFSET  = -1;
   aMeses : Array [1..12] Of Byte = (31,28,31,30,31,30,31,31,30,31,30,31);
Var
   Yr,Mn,n1,n2 : Integer;
Begin
yr := nAno;
mn := nMes;

If (mn<3) then
   Begin
   Inc (mn,12);
   Dec (yr)
   End;

n1 := (26*(mn + 1)) div 10 ;
n2 := ((125*LongInt(yr)) div 100) ;

nDiaS := ((nDia + n1 + n2 - (yr div 100) + (yr div 400) + ZELLER_OFFSET) mod 7) ;

If ((nAno mod 4)=0) and ((nAno mod 40)<>0) then aMeses[2] := 29
                                           else aMeses[2] := 28;
nTotDias := aMeses[nMes]
End;


Function ExtractDate (S:String):String;
Begin
ExtractDate := Copy(S,7,2)+'/'+Copy(S,5,2)+'/'+Copy(S,1,4);
End;


Function ExtractTime (S:String):String;
Begin
ExtractTime := Copy(S,9,2)+'h'+Copy(S,11,2)+'min'+Copy(S,13,2)+'s';
End;

Function ComputeDTime (Date,Time:String):String;
Begin
ComputeDTime := Copy(Date,7,4)+Copy(Date,4,2)+Copy(Date,1,2)+Copy(Time,1,2)+Copy(Time,4,2)+Copy(Time,9,2);
End;


 function HexToInt(S: String): LongInt;
 const
   cHexDig: String[16] = '0123456789ABCDEF';
 var
   nMult,
   nAux: LongInt;
   I, nPos: Integer;
 begin
   S := AllTrim(S);
   nAux := 0;
   nMult := 1;
   for i := Length(S) downto 1 do begin
     nPos := Pos(S[i], cHexDig);
     if nPos<>0 then
       nAux := nAux+(nPos-1)*nMult;
     nMult := nMult*16;
   end;
   HexToInt := nAux;
 end;

 function ToHex(nNumber, nSize: LongInt): String;
 const
   cCharHex: String[16] = '0123456789ABCDEF';
 var
   cAux: String;
   I: Integer;
 begin
   cAux := '';
   for i := 1 to nSize do begin
     cAux := cCharHex[nNumber mod 16+1]+cAux;
     nNumber := nNumber div 16;
   end;
   ToHex := cAux;
 end;


End.