unit
  UVDP;

interface

uses
  UDIBSurf;

  const
    aVDPInitSMS: array[0..15] of byte =
     ($00,$60,$0E,$00,$00,$7F,$00,$00,
      $00,$00,$FF,$00,$00,$00,$00,$00);
    aVDPInitColeco: array[0..15] of byte =
     ($00,$10,$00,$00,$00,$00,$00,$00,
      $00,$00,$00,$00,$00,$00,$00,$00);
    aMSK: array[0..3, 0..3] of LongInt =
     (($7F,$00,$3F,$00),  (* SCR 0:TEXT 40x24  *)
      ($7F,$FF,$3F,$FF),  (* SCR 1:TEXT 32x24  *)
      ($7F,$80,$3C,$FF),  (* SCR 2:BLK 256x192 *)
      ($7F,$00,$3F,$FF)); (* SCR 3:64x48x16    *)


  type
    TEmulObject = object
      constructor Init(P: Pointer; N: Integer);
      destructor  Done; virtual;
    end;

    TVDP=object(TEmulObject)
      nRAddr, nWAddr: Word;
      VRAM: PChar;
      aVDP: array[0..15] of Byte;
      nCurLine, nMinLine, nMaxLine: Integer;
      nColTab,
      nChrTab,
      nSprTab,
      nSprGen,
      nChrGen,
      nChrPal,
      nSprPal: Word;
      nVKey, nVR,
      nVDPStatus,
      nScrMode,
      nFGColor,
      nBGColor: Byte;
      nScrollX, nScrollY: Byte;
      constructor Init(P: Pointer; N: Integer);
      procedure   OutByte(nAddr, nValue: Word);
      procedure   SetScroll;
      function    Interrupt: LongInt;
      function    ReadVRAM: Byte;
      procedure   WriteVRAM(nValue: Byte);
      destructor  Done; virtual;
    end;

implementation

constructor TEmulObject.Init; begin end;
destructor TEmulObject.Done; begin end;


constructor TVDP.Init;
var
  I: Integer;
begin
  GetMem(VRAM, 65500);
  for i := 0 to N-1 do
    aVDP[i] := Byte(PChar(P)[i]);
  nVKey := 1;
  nVDPStatus := $9F;
  nFGColor := 0;
  nBGColor := 0;
  nScrMode := 0;
  nChrTab := 0;
  nColTab := 0;
  nChrGen := 0;
  nSprTab := 0;
  nSprGen := 0;
  nChrTab := ((LongInt(aVDP[2]) and aMSK[nScrMode, 0]) shl 10);
  nChrGen := ((LongInt(aVDP[4]) and aMSK[nScrMode, 2]) shl 11);
  nColTab := ((LongInt(aVDP[3]) and aMSK[nScrMode, 1]) shl 6);
  nSprTab := ((LongInt(aVDP[5]) and aMSK[nScrMode, 3]) shl 7);
  nSprGen := (LongInt(aVDP[6]) shl 11);
  nWAddr := 0;
end;

procedure TVDP.OutByte;
var
  J : LongInt;
begin
  case nAddr of
     0: begin
       case (((nValue and $0E) shr 1) or (aVDP[1] and $18)) of
         $10: J := 0;
         $00: J := 1;
         $01: J := 2;
         $08: J := 3;
         else J := nScrMode;
       end;
       if (J <> nScrMode) then begin
         nChrTab := ((LongInt(aVDP[2]) and aMSK[J, 0]) shl 10);
         nChrGen := ((LongInt(aVDP[4]) and aMSK[J, 2]) shl 11);
         nColTab := ((LongInt(aVDP[3]) and aMSK[J, 1]) shl 6);
         nSprTab := ((LongInt(aVDP[5]) and aMSK[J, 3]) shl 7);
         nSprGen := (LongInt(aVDP[6]) shl 11);
         nScrMode := J;
         { Iniciar Cores e Modo Grafico }
       end
     end;
     1: begin
       case (((aVDP[0] and $0E) shr 1) or (nValue and $18)) of
         $10: J := 0;
         $00: J := 1;
         $01: J := 2;
         $08: J := 3;
         else J := nScrMode;
       end;
       if (J <> nScrMode) then begin
         nChrTab := ((LongInt(aVDP[2]) and aMSK[J, 0]) shl 10);
         nChrGen := ((LongInt(aVDP[4]) and aMSK[J, 2]) shl 11);
         nColTab := ((LongInt(aVDP[3]) and aMSK[J, 1]) shl 6);
         nSprTab := ((LongInt(aVDP[5]) and aMSK[J, 3]) shl 7);
         nSprGen := (LongInt(aVDP[6]) shl 11);
         nScrMode := J;
         { Iniciar Cores e Modo Grafico }
       end;
     end;
     2: nChrTab := ((LongInt(nValue) and aMSK[nScrMode, 0]) shl 10);
     3: nColTab := ((LongInt(nValue) and aMSK[nScrMode, 1]) shl 6);
     4: nChrGen := ((LongInt(nValue) and aMSK[nScrMode, 2]) shl 11);
     5: nSprTab := ((LongInt(nValue) and aMSK[nScrMode, 3]) shl 7);
     6: begin
       nValue := nValue and $3F;
       nSprGen := (LongInt(nValue) shl 11);
     end;
     7: begin
       nFGColor := nValue shr 4;
       nBGColor := nValue and $0F;
       { Inializar Cores }
     end;
  end;
  aVDP[nAddr] := nValue;
end;


procedure TVDP.SetScroll;
begin
  if ((aVDP[0] and $40)<>0) and (nCurLine<(nMinLine+16)) then
    nScrollX := 0
  else
    nScrollX := 256-aVDP[8];
  nScrollY := aVDP[9];
end;

function TVDP.Interrupt: LongInt;
var
  FC,BC: Byte;
  C, PGT, CLT, K, I, J, X, Y, T, S, P, N: Integer;
const
  WIDTH=256;
  HEIGHT=256;
begin
  if (nVKey=0) then begin
    Interrupt := $01000000;
    Exit;
  end;


  case nScrMode of
    1: begin
      T := nChrTab;
      for Y := 24 downto 0 do begin
        for x := 0 to 31 do begin
          S := nChrGen+(Byte(VRAM[T]) shr 3);
          if Byte(VRAM[T])<>0 then begin
            nScrMode := 1;
          end;
          BC := 1;{Byte(VRAM[nColTab+(Byte(VRAM[T]) shr 3)]);}
          FC := 7; {BC+1;}
          for i := 0 to 7 do begin
            K := Byte(VRAM[S]);
            for j := 0 to 7 do begin
              if (K and 1)<>0 then
                SetPixelSafe(8*X+i, 8*Y+j, FC)
              else
                SetPixelSafe(8*X+i, 8*Y+j, BC);
              k := k shr 1;
            end;
            S := S+1;
          end;
          T := T+1;
        end;
      end;
    end;
    2: begin
      PGT := nChrGen;
      CLT := nColTab;
      T := nChrTab;

      for n := 0 to 2 do begin
        for y := 0 to 7 do begin
          for x := 0 to 31 do begin
            S := PGT+(LongInt(VRAM[T]) shl 3);
            C := CLT+(LongInt(VRAM[T]) shl 3);
            for i := 0 to 7 do begin
              K := Byte(VRAM[S]);
              FC := 1;
              BC := 7;
              for j := 0 to 7 do begin
                if (K and 1)<>0 then
                  SetPixelSafe(8*X+7-j, n*64+8*Y+i, FC)
                else
                  SetPixelSafe(8*X+7-j, n*64+8*Y+i, BC);
                k := k shr 1;
              end;
              C := C+1;
              S := S+1;
            end;
            T := T+1;
          end;
        end;
        PGT := PGT+$800;
        CLT := CLT+$800;
      end;
    end;
  end;


  nVDPStatus := (nVDPStatus and $DF) or $80;

  if (aVDP[1] and $20)=0 then
    Interrupt := $00000066
  else
    Interrupt :=$01000000;

end;


function TVDP.ReadVRAM: Byte;
begin
  ReadVRAM := Byte(VRAM[nRAddr]);
  nRAddr := (nRAddr+1) and $3FFF;
end;

procedure TVDP.WriteVRAM(nValue: Byte);
begin
  Byte(VRAM[nWAddr]) := nValue;
  nWAddr := (nWAddr+1) and $3FFF;
end;


destructor TVDP.Done;
begin
  FreeMem(VRAM, 65500);
end;


end.