unit uz80dsm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons;

type
  TString16 = String[16];
  AString16 = array[1..1] of TString16;
  PString16 = ^AString16;
  TfrmDisasm = class(TForm)
    Panel1: TPanel;
    pnlShow: TPanel;
    pbShow: TPaintBox;
    Panel2: TPanel;
    Panel3: TPanel;
    bbHide: TBitBtn;
    sbShow: TScrollBar;
    procedure bbHideClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbShowPaint(Sender: TObject);
    procedure sbShowChange(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    oStrings: PString16;
    nTotInstr: Integer;
    nCabe: Integer;
  public
    { Public declarations }
    procedure ClearStruct;
    procedure Disasm(P: Pointer; nMaxROM: Integer);
    procedure SetPos(nX, nY, nW, nH: Integer);
    function IsCleared: Boolean;
  end;

var
  frmDisasm: TfrmDisasm;

implementation

uses
  UDisasm, uprogr;

{$R *.DFM}

procedure TfrmDisasm.bbHideClick(Sender: TObject);
begin
  Visible := false;
end;

procedure TfrmDisasm.FormCreate(Sender: TObject);
begin
  oStrings := nil;
  nTotInstr := 0;
  nCabe := pbShow.Height div 12;
  sbShow.LargeChange := nCabe;
end;

procedure TfrmDisasm.ClearStruct;
begin
  if oStrings<>nil then begin
    FreeMem(oStrings, nTotInstr*SizeOf(TString16));
  end;
  oStrings := nil;
  nTotInstr := 0;
end;

procedure TfrmDisasm.Disasm(P: Pointer; nMaxROM: Integer);
const
  aBit: array[1..4] of byte = ($80, $40, $20, $10);
var
  nTInstr, nAddr: Integer;
  aROMInstr: array[0..3] of Byte;
  nPos, nIndex, I, J, K: Integer;
  lFound, lLixo: Boolean;
  cAux1,
  cAux: String;
  nPorc, nOldPorc: Real;
begin
  frmProgress.Show;
  frmProgress.gProgress.Visible := true;
  frmProgress.gProgress.MinValue := 0;
  frmProgress.gProgress.MaxValue := 100;
  frmProgress.gProgress.Progress := 0;

  nOldPorc := -1;

  for k := 1 to 2 do begin
    nAddr := 0;
    if k=1 then begin
      nTInstr := 0
    end
    else begin
      nTotInstr := 0;
      GetMem(oStrings, nTInstr*SizeOf(TString16));
    end;

    nTotInstr := 0;
    while nAddr<=nMaxROM do begin

      nPorc := k*nAddr/nMaxRom/2*100;
      if nPorc>nOldPorc then begin
        nOldPorc := nPorc+5;

        frmProgress.gProgress.Progress := Round(nPorc);
        frmProgress.gProgress.Refresh;
        {Application.ProcessMessages;}
      end;



      aROMInstr[0] := Byte(PChar(P)[nAddr+0]);
      aROMInstr[1] := Byte(PChar(P)[nAddr+1]);
      aROMInstr[2] := Byte(PChar(P)[nAddr+2]);
      aROMInstr[3] := Byte(PChar(P)[nAddr+3]);
      lFound := false;
      cAux := '';
      for i := 1 to MAX_Z80_INSTR do begin
        with aZ80Instr[i] do begin
          lFound := true;
          for j := 1 to nMask and $0F do
            if (nMask and aBit[j])<>0 then begin
              if aROMInstr[j-1]<>aInstr[j-1] then lFound := false;
            end
            else begin
              cAux := IntToHex(aROMInstr[j-1], 2)+cAux;
            end;
          nIndex := i;
          if lFound then begin
            lLixo := true;
            break;
          end;
        end;
      end;

      if k=1 then
        Inc(nTInstr)
      else begin
        Inc(nTotInstr);

        oStrings^[nTotInstr] := IntToHex(nAddr, 4)+aZ80Instr[nIndex].cInstr;
        nPos := Pos('##', oStrings^[nTotInstr]);
        if nPos<>0 then begin
          oStrings^[nTotInstr][nPos+0] := cAux[1];
          oStrings^[nTotInstr][nPos+1] := cAux[2];
          Delete(cAux, 1, 2);

          nPos := Pos('##', oStrings^[nTotInstr]);
          if nPos<>0 then begin
            oStrings^[nTotInstr][nPos+0] := cAux[1];
            oStrings^[nTotInstr][nPos+1] := cAux[2];
          end;
        end;
      end;

      if lFound or lLixo then
        nAddr := nAddr+(aZ80Instr[nIndex].nMask and $0F)
      else
        Inc(nAddr);
    end;
  end;
  sbShow.Min := 0;
  sbShow.Max := nTotInstr-nCabe;
  frmProgress.gProgress.Visible := false;
  frmProgress.Hide;
end;

procedure TfrmDisasm.pbShowPaint(Sender: TObject);
var
  I: Integer;
begin
  if oStrings=nil then exit;

  with pbShow.Canvas do begin
    Brush.Style := BSSOLID;
    Brush.Color := CLWHITE;
    FillRect(pbShow.ClientRect);

    Font.Name := 'ARIAL';
    Font.Style := [];
    Font.Size := 8;

    Brush.Style := BSCLEAR;
    for i := 1 to nCabe do begin
      if (sbShow.Position+i)<=nTotInstr then begin
        Font.Style := [FSBOLD];
        TextOut(004, 12*(i-1), Copy(oStrings^[sbShow.Position+i], 1, 4));

        Font.Style := [];
        TextOut(040, 12*(i-1), Copy(oStrings^[sbShow.Position+i], 5, 255));
      end;
    end;
  end;
end;

procedure TfrmDisasm.SetPos(nX, nY, nW, nH: Integer);
begin
  if nX>=0 then Left := nX;
  if nY>=0 then Top := nY;
  if nW>=0 then Width := nW;
  if nH>=0 then Height := nH;
end;

procedure TfrmDisasm.sbShowChange(Sender: TObject);
begin
  pbShow.Refresh;
end;

procedure TfrmDisasm.FormResize(Sender: TObject);
begin
  sbShow.Height := Panel3.Height-8;
  nCabe := pbShow.Height div 12;
  sbShow.LargeChange := nCabe;
  sbShow.Max := nTotInstr-nCabe;
end;

function TfrmDisasm.IsCleared: Boolean;
begin
  IsCleared := oStrings = nil;
end;

end.
